@@ -170,7 +170,7 @@ totemPlugin::~totemPlugin ()
 		dbus_g_proxy_disconnect_signal (mBusProxy,
 						"NameOwnerChanged",
 						G_CALLBACK (NameOwnerChangedCallback),
-						NS_REINTERPRET_CAST (void*, this));
+						reinterpret_cast<void*>(this));
 		g_object_unref (mBusProxy);
 		mBusProxy = NULL;
 	}
@@ -349,7 +349,7 @@ totemPlugin::ViewerFork ()
 	/* Don't wait forever! */
 	const PRUint32 kViewerTimeout = 30 * 1000; /* ms */
 	nsresult rv = mTimer->InitWithFuncCallback (ViewerForkTimeoutCallback,
-						    NS_REINTERPRET_CAST (void*, this),
+						    reinterpret_cast<void*>(this),
 						    kViewerTimeout,
 						    nsITimer::TYPE_ONE_SHOT);
 	if (NS_FAILED (rv)) {
@@ -424,7 +424,7 @@ totemPlugin::ViewerSetup ()
 	dbus_g_proxy_connect_signal (mViewerProxy,
 				     "ButtonPress",
 				     G_CALLBACK (ButtonPressCallback),
-				     NS_REINTERPRET_CAST (void*, this),
+				     reinterpret_cast<void*>(this),
 				     NULL);
 
 	dbus_g_proxy_add_signal (mViewerProxy,
@@ -433,7 +433,7 @@ totemPlugin::ViewerSetup ()
 	dbus_g_proxy_connect_signal (mViewerProxy,
 				     "StopStream",
 				     G_CALLBACK (StopStreamCallback),
-				     NS_REINTERPRET_CAST (void*, this),
+				     reinterpret_cast<void*>(this),
 				     NULL);
 
 	if (mHidden) {
@@ -472,11 +472,11 @@ totemPlugin::ViewerCleanup ()
 		dbus_g_proxy_disconnect_signal (mViewerProxy,
 						"ButtonPress",
 						G_CALLBACK (ButtonPressCallback),
-						NS_REINTERPRET_CAST (void*, this));
+						reinterpret_cast<void*>(this));
 		dbus_g_proxy_disconnect_signal (mViewerProxy,
 						"StopStream",
 						G_CALLBACK (StopStreamCallback),
-						NS_REINTERPRET_CAST (void*, this));
+						reinterpret_cast<void*>(this));
 
 		g_object_unref (mViewerProxy);
 		mViewerProxy = NULL;
@@ -519,7 +519,7 @@ totemPlugin::ViewerSetWindow ()
 		dbus_g_proxy_begin_call (mViewerProxy,
 					 "SetWindow",
 					 ViewerSetWindowCallback,
-					 NS_REINTERPRET_CAST (void*, this),
+					 reinterpret_cast<void*>(this),
 					 NULL,
 #if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
 					 G_TYPE_STRING, mControls.get (),
@@ -767,7 +767,7 @@ totemPlugin::RequestStream (PRBool aForceViewer)
 			dbus_g_proxy_begin_call (mViewerProxy,
 						 "OpenStream",
 						 ViewerOpenStreamCallback,
-						 NS_REINTERPRET_CAST (void*, this),
+						 reinterpret_cast<void*>(this),
 						 NULL,
 						 G_TYPE_STRING, spec.get (),
 						 G_TYPE_STRING, baseSpec.get (),
@@ -777,7 +777,7 @@ totemPlugin::RequestStream (PRBool aForceViewer)
 			dbus_g_proxy_begin_call (mViewerProxy,
 						 "OpenURI",
 						 ViewerOpenURICallback,
-						 NS_REINTERPRET_CAST (void*, this),
+						 reinterpret_cast<void*>(this),
 						 NULL,
 						 G_TYPE_STRING, spec.get (),
 						 G_TYPE_STRING, baseSpec.get (),
@@ -816,7 +816,7 @@ totemPlugin::NameOwnerChangedCallback (DBusGProxy *proxy,
 				       const char *aNewOwner,
 				       void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	plugin->NameOwnerChanged (aName, aOldOwner, aNewOwner);
 }
@@ -825,7 +825,7 @@ totemPlugin::NameOwnerChangedCallback (DBusGProxy *proxy,
 totemPlugin::ViewerForkTimeoutCallback (nsITimer *aTimer,
 				        void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("ViewerForkTimeoutCallback");
 
@@ -842,7 +842,7 @@ totemPlugin::ButtonPressCallback (DBusGProxy *proxy,
 				  guint aButton,
 			          void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("ButtonPress signal received");
 
@@ -853,7 +853,7 @@ totemPlugin::ButtonPressCallback (DBusGProxy *proxy,
 totemPlugin::StopStreamCallback (DBusGProxy *proxy,
 			         void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("StopStream signal received");
 
@@ -865,7 +865,7 @@ totemPlugin::ViewerSetWindowCallback (DBusGProxy *aProxy,
 				      DBusGProxyCall *aCall,
 				      void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("SetWindow reply");
 
@@ -889,7 +889,7 @@ totemPlugin::ViewerOpenStreamCallback (DBusGProxy *aProxy,
 				       DBusGProxyCall *aCall,
 				       void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("OpenStream reply");
 
@@ -936,7 +936,7 @@ totemPlugin::ViewerOpenURICallback (DBusGProxy *aProxy,
 				    DBusGProxyCall *aCall,
 				    void *aData)
 {
-	totemPlugin *plugin = NS_REINTERPRET_CAST (totemPlugin*, aData);
+	totemPlugin *plugin = reinterpret_cast<totemPlugin*>(aData);
 
 	D ("OpenURI reply");
 
@@ -983,67 +983,6 @@ totemPlugin::GetRealMimeType (const char *mimetype,
 	D ("Real mime-type for '%s' not found", mimetype);
 }
 
-PRBool
-totemPlugin::IsMimeTypeSupported (const char *mimetype,
-				  const char *url)
-{
-	NS_ENSURE_TRUE (mimetype, PR_FALSE);
-
-	return PR_TRUE;
-
-	/* Disabled for the benefit of:
-	 * http://bugzilla.gnome.org/show_bug.cgi?id=418387 */
-#if 0
-#if defined(TOTEM_NARROWSPACE_PLUGIN) || defined(TOTEM_GMP_PLUGIN) || defined(TOTEM_MULLY_PLUGIN)
-	/* We can always play those image types */
-	if (strcmp (mimetype, "image/jpeg") == 0)
-		return PR_TRUE;
-	if (strcmp (mimetype, "image/gif") == 0)
-		return PR_TRUE;
-#endif /* TOTEM_NARROWSPACE_PLUGIN || TOTEM_GMP_PLUGIN || TOTEM_MULLY_PLUGIN */
-
-	/* Stupid web servers will do that */
-	if (strcmp (mimetype, GNOME_VFS_MIME_TYPE_UNKNOWN) == 0)
-		return PR_TRUE;
-
-	const totemPluginMimeEntry *mimetypes;
-	PRUint32 count;
-	totemScriptablePlugin::PluginMimeTypes (&mimetypes, &count);
-
-	for (PRUint32 i = 0; i < count; ++i) {
-		if (strcmp (mimetypes[i].mimetype, mimetype) == 0)
-			return PR_TRUE;
-	}
-
-	/* Not supported? Probably a broken webserver */
-	const char *guessed = gnome_vfs_get_mime_type_for_name (url);
-
-	D ("Guessed mime-type '%s' for '%s'", guessed, url);
-	for (PRUint32 i = 0; i < count; ++i) {
-		if (strcmp (mimetypes[i].mimetype, guessed) == 0)
-			return PR_TRUE;
-	}
-
-	/* Still unsupported? Try to get it without the arguments
-	 * passed to the script */
-	const char *s = strchr (url, '?');
-	if (s == NULL)
-		return PR_FALSE;
-
-	char *no_args = g_strndup (url, s - url);
-	guessed = gnome_vfs_get_mime_type_for_name (no_args);
-	D ("Guessed mime-type '%s' for '%s' without the arguments", guessed, url);
-	g_free (no_args);
-
-	for (PRUint32 i = 0; i < count; ++i) {
-		if (strcmp (mimetypes[i].mimetype, guessed) == 0)
-			return PR_TRUE;
-	}
-
-	return PR_FALSE;
-#endif
-}
-
 PRBool
 totemPlugin::IsSchemeSupported (nsIURI *aURI)
 {
@@ -1521,8 +1460,8 @@ totemPlugin::Init (NPMIMEType mimetype,
 	NPError err;
 	err = CallNPN_GetValueProc (sNPN.getvalue,
 				    mInstance, NPNVserviceManager,
-				    NS_REINTERPRET_CAST (void *,
-							 NS_REINTERPRET_CAST (void **, &mServiceManager)));
+				    reinterpret_cast<void *>
+                        (reinterpret_cast<void **>(&mServiceManager)));
 	if (err != NPERR_NO_ERROR || !mServiceManager) {
 		D ("Failed to get the service manager");
 		return NPERR_GENERIC_ERROR;
@@ -1531,7 +1470,7 @@ totemPlugin::Init (NPMIMEType mimetype,
 	nsresult rv;
 	rv = mServiceManager->GetServiceByContractID (NS_IOSERVICE_CONTRACTID,
 						      NS_GET_IID (nsIIOService),
-						      NS_REINTERPRET_CAST (void **, &mIOService));
+						      reinterpret_cast<void **>(&mIOService));
 	if (NS_FAILED (rv) || !mIOService) {
 		D ("Failed to get IO service");
 		return NPERR_GENERIC_ERROR;
@@ -1539,8 +1478,8 @@ totemPlugin::Init (NPMIMEType mimetype,
 
 	err = CallNPN_GetValueProc (sNPN.getvalue,
 				    mInstance, NPNVDOMElement,
-				    NS_REINTERPRET_CAST (void *,
-						         NS_REINTERPRET_CAST (void **, &mPluginDOMElement)));
+				    reinterpret_cast<void *>
+                        (reinterpret_cast<void **>(&mPluginDOMElement)));
 	if (err != NPERR_NO_ERROR || !mPluginDOMElement) {
 		D ("Failed to get our DOM Element");
 		return NPERR_GENERIC_ERROR;
@@ -1598,7 +1537,7 @@ totemPlugin::Init (NPMIMEType mimetype,
         rv = compMan->CreateInstanceByContractID (NS_TIMER_CONTRACTID,
                                                   nsnull,
                                                   NS_GET_IID (nsITimer),
-                                                  NS_REINTERPRET_CAST (void **, &mTimer));
+                                                  reinterpret_cast<void **>(&mTimer));
         if (NS_FAILED (rv) || !mTimer) {
                 D ("Failed to create timer: rv=%x", rv);
                 return NPERR_GENERIC_ERROR;
@@ -1630,7 +1569,7 @@ totemPlugin::Init (NPMIMEType mimetype,
 	dbus_g_proxy_connect_signal (mBusProxy,
 				     "NameOwnerChanged",
 				     G_CALLBACK (NameOwnerChangedCallback),
-				     NS_REINTERPRET_CAST (void*, this),
+				     reinterpret_cast<void*>(this),
 				     NULL);
 
 	/* Find the "real" mime-type */
@@ -1967,15 +1906,6 @@ totemPlugin::NewStream (NPMIMEType type,
 	}
 #endif
 
-	if (!IsMimeTypeSupported (type, stream->url)) {
-		D ("Unsupported mimetype, aborting stream");
-
-		return CallNPN_DestroyStreamProc (sNPN.destroystream,
-						  mInstance,
-						  stream,
-						  NPRES_DONE);
-	}
-
 	/* FIXME: assign the stream URL to mRequestURI ? */
 
 	if (g_str_has_prefix (stream->url, "file://")) {
@@ -2232,7 +2162,7 @@ totemPlugin::GetScriptable (void *_retval)
 	}
 
 	nsresult rv = mScriptable->QueryInterface (NS_GET_IID (nsISupports),
-						   NS_REINTERPRET_CAST (void **, _retval));
+						   reinterpret_cast<void **>(_retval));
 
 	return NS_SUCCEEDED (rv) ? NPERR_NO_ERROR : NPERR_GENERIC_ERROR;
 }
