@@ -26,7 +26,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "totem-remote.h"
 #include "totem-options.h"
 #include "totem-uri.h"
 #include "bacon-video-widget.h"
@@ -50,7 +49,7 @@ const GOptionEntry options[] = {
 	{"quit", '\0', 0, G_OPTION_ARG_NONE, &optionstate.quit, N_("Quit"), NULL},
 	{"enqueue", '\0', 0, G_OPTION_ARG_NONE, &optionstate.enqueue, N_("Enqueue"), NULL},
 	{"replace", '\0', 0, G_OPTION_ARG_NONE, &optionstate.replace, N_("Replace"), NULL},
-	{"printplaying", '\0', 0, G_OPTION_ARG_NONE, &optionstate.printplaying, "Print playing movie", NULL}, //FIXME translate
+	{"printplaying", '\0', 0, G_OPTION_ARG_NONE, &optionstate.printplaying, N_("Print playing movie"), NULL},
 	{"seek", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_INT64, &optionstate.seek, N_("Seek"), NULL},
 	{"playlist-idx", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_DOUBLE, &optionstate.playlistidx, N_("Playlist index"), NULL},
 	{G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &optionstate.filenames, N_("Movies to play"), NULL},
@@ -60,9 +59,6 @@ const GOptionEntry options[] = {
 void
 totem_options_process_late (Totem *totem, const TotemCmdLineOptions* options)
 {
-	if (options->fullscreen) 
-		totem_action_fullscreen_toggle (totem);
-
 	if (options->togglecontrols) 
 		totem_action_toggle_controls (totem);
 
@@ -74,18 +70,16 @@ totem_options_process_late (Totem *totem, const TotemCmdLineOptions* options)
 }
 
 void
-totem_options_process_early (GConfClient *gc, const TotemCmdLineOptions* options)
+totem_options_process_early (Totem *totem, const TotemCmdLineOptions* options)
 {
-	if (options->debug)
-	{
-		gconf_client_set_bool (gc, GCONF_PREFIX"/debug",
-				TRUE, NULL);
-	} else if (options->quit) 
-	{
+	if (options->quit) {
 		/* If --quit is one of the commands, just quit */
 		gdk_notify_startup_complete ();
 		exit (0);
 	}
+
+	gconf_client_set_bool (totem->gc, GCONF_PREFIX"/debug",
+			       options->debug, NULL);
 }
 
 static void
@@ -152,7 +146,8 @@ totem_options_process_for_server (BaconMessageConnection *conn,
 	{
 		char *line, *full_path;
 		full_path = totem_create_full_path (options->filenames[i]);
-		line = g_strdup_printf ("%03d %s", default_action, full_path);
+		line = g_strdup_printf ("%03d %s", default_action,
+					full_path ? full_path : options->filenames[i]);
 		bacon_message_connection_send (conn, line);
 		/* Even if the default action is replace, we only want to replace with the
 		   first file.  After that, we enqueue. */
@@ -221,6 +216,7 @@ totem_options_process_for_server (BaconMessageConnection *conn,
 		char *line;
 		line = totem_option_create_line (TOTEM_REMOTE_COMMAND_SHOW);
 		bacon_message_connection_send (conn, line);
+		g_free (line);
 		return;
 	}
 
@@ -229,5 +225,6 @@ totem_options_process_for_server (BaconMessageConnection *conn,
 		bacon_message_connection_send (conn, l->data);
 		g_free (l->data);
 	}
+	g_list_free (commands);
 }
 
