@@ -27,7 +27,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <glade/glade.h>
 #include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -43,6 +42,13 @@
 static void ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle);
 static gboolean totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl, const char *display_name);
 
+/* Callback function for GtkBuilder */
+void totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist);
+void totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist);
+void playlist_remove_button_clicked (GtkWidget *button, TotemPlaylist *playlist);
+void totem_playlist_up_files (GtkWidget *widget, TotemPlaylist *playlist);
+void totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist);
+
 typedef gboolean (*PlaylistCallback) (TotemPlaylist *playlist, const char *mrl,
 		gpointer data);
 
@@ -59,7 +65,7 @@ typedef struct {
 
 struct TotemPlaylistPrivate
 {
-	GladeXML *xml;
+	GtkBuilder *xml;
 
 	GtkWidget *treeview;
 	GtkTreeModel *model;
@@ -149,7 +155,7 @@ static void totem_playlist_init       (TotemPlaylist      *playlist);
 
 static void init_treeview (GtkWidget *treeview, TotemPlaylist *playlist);
 
-#define totem_playlist_unset_playing(x) totem_playlist_set_playing(x, FALSE)
+#define totem_playlist_unset_playing(x) totem_playlist_set_playing(x, TOTEM_PLAYLIST_STATUS_NONE)
 
 G_DEFINE_TYPE(TotemPlaylist, totem_playlist, GTK_TYPE_VBOX)
 
@@ -262,72 +268,17 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 	return filename_for_display;
 }
 
-static gboolean
-totem_playlist_is_media (const char *mrl)
-{
-	if (mrl == NULL)
-		return FALSE;
-
-	if (g_str_has_prefix (mrl, "cdda:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (mrl, "dvd:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (mrl, "vcd:") != FALSE)
-		return TRUE;
-	if (g_str_has_prefix (mrl, "cd:") != FALSE)
-		return TRUE;
-
-	return FALSE;
-}
-
 static void
 totem_playlist_update_save_button (TotemPlaylist *playlist)
 {
 	GtkWidget *button;
 	gboolean state;
 
-	button = glade_xml_get_widget (playlist->_priv->xml, "save_button");
+	button = GTK_WIDGET (gtk_builder_get_object (playlist->_priv->xml, "save_button"));
 	state = (!playlist->_priv->disable_save_to_disk) && (PL_LEN != 0);
 	gtk_widget_set_sensitive (button, state);
 }
 
-static char*
-totem_playlist_create_full_path (const char *path)
-{
-	char *retval, *curdir, *curdir_withslash, *escaped;
-
-	g_return_val_if_fail (path != NULL, NULL);
-
-	if (strstr (path, "://") != NULL)
-		return g_strdup (path);
-	if (totem_playlist_is_media (path) != FALSE)
-		return g_strdup (path);
-
-	if (path[0] == '/')
-	{
-		escaped = gnome_vfs_escape_path_string (path);
-
-		retval = g_strdup_printf ("file://%s", escaped);
-		g_free (escaped);
-		return retval;
-	}
-
-	curdir = g_get_current_dir ();
-	escaped = gnome_vfs_escape_path_string (curdir);
-	curdir_withslash = g_strdup_printf ("file://%s%s",
-			escaped, G_DIR_SEPARATOR_S);
-	g_free (escaped);
-	g_free (curdir);
-
-	escaped = gnome_vfs_escape_path_string (path);
-	retval = gnome_vfs_uri_make_full_from_relative
-		(curdir_withslash, escaped);
-	g_free (curdir_withslash);
-	g_free (escaped);
-
-	return retval;
-}
-
 static void
 totem_playlist_save_get_iter_func (GtkTreeModel *model,
 		GtkTreeIter *iter, char **uri, char **title,
@@ -436,8 +387,7 @@ drop_cb (GtkWidget     *widget,
 		filename = totem_create_full_path (p->data);
 		title = NULL;
 
-		if (info == 1)
-		{
+		if (info == 1) {
 			g_free (p->data);
 			p = p->next;
 			if (p != NULL) {
@@ -448,7 +398,7 @@ drop_cb (GtkWidget     *widget,
 			}
 		}
 
-		totem_playlist_add_mrl (playlist, p->data, title);
+		totem_playlist_add_mrl (playlist, filename ? filename : p->data, title);
 
 		g_free (filename);
 		g_free (p->data);
@@ -576,7 +526,7 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 		GtkTreeIter iter;
 		char *index;
 		GtkTreePath *path;
-		gboolean playing;
+		TotemPlaylistStatus playing;
 
 		index = g_strdup_printf ("%d", i);
 		if (gtk_tree_model_get_iter_from_string
@@ -589,9 +539,8 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 		g_free (index);
 
 		gtk_tree_model_get (playlist->_priv->model, &iter, PLAYING_COL, &playing, -1);
-		if (!playing) {
+		if (playing == TOTEM_PLAYLIST_STATUS_NONE)
 			continue;
-		}
 
 		/* Only emit the changed signal if we changed the ->current */
 		path = gtk_tree_path_new_from_indices (i, -1);
@@ -680,11 +629,11 @@ selection_changed (GtkTreeSelection *treeselection, TotemPlaylist *playlist)
 	GtkWidget *remove_button, *up_button, *down_button;
 	gboolean sensitivity;
 
-	remove_button = glade_xml_get_widget (playlist->_priv->xml,
-			"remove_button");
-	up_button = glade_xml_get_widget (playlist->_priv->xml, "up_button");
-	down_button = glade_xml_get_widget (playlist->_priv->xml,
-			"down_button");
+	remove_button = GTK_WIDGET (gtk_builder_get_object (playlist->_priv->xml,
+			"remove_button"));
+	up_button = GTK_WIDGET (gtk_builder_get_object (playlist->_priv->xml, "up_button"));
+	down_button = GTK_WIDGET (gtk_builder_get_object (playlist->_priv->xml,
+			"down_button"));
 
 	if (gtk_tree_selection_has_selected (treeselection))
 		sensitivity = TRUE;
@@ -726,7 +675,7 @@ update_current_from_playlist (TotemPlaylist *playlist)
 	return TRUE;
 }
 
-static void
+void
 totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
 	GSList *filenames, *l;
@@ -869,7 +818,7 @@ playlist_remove_files (TotemPlaylist *playlist)
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->_priv->treeview));
 }
 
-static void
+void
 playlist_remove_button_clicked (GtkWidget *button, TotemPlaylist *playlist)
 {
 	playlist_remove_files (playlist);
@@ -939,7 +888,7 @@ totem_playlist_save_add_format_combo_box (GtkFileChooser *fc)
 	return combo_box;
 }
 
-static void
+void
 totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
 	GtkWidget *fs, *combo_box;
@@ -1112,13 +1061,13 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 			NULL);
 }
 
-static void
+void
 totem_playlist_up_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
 	totem_playlist_move_files (playlist, TRUE);
 }
 
-static void
+void
 totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
 	totem_playlist_move_files (playlist, FALSE);
@@ -1159,14 +1108,25 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 
 static void
 set_playing_icon (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-		GtkTreeModel *model, GtkTreeIter *iter, TotemPlaylist *playlist)
+		  GtkTreeModel *model, GtkTreeIter *iter, TotemPlaylist *playlist)
 {
-	gboolean playing;
+	TotemPlaylistStatus playing;
+	const char *stock_id;
 
 	gtk_tree_model_get (model, iter, PLAYING_COL, &playing, -1);
 
-	g_object_set (renderer, "icon-name",
-		      playing ? "audio-volume-medium" : NULL, NULL);
+	switch (playing) {
+		case TOTEM_PLAYLIST_STATUS_PLAYING:
+			stock_id = GTK_STOCK_MEDIA_PLAY;
+			break;
+		case TOTEM_PLAYLIST_STATUS_PAUSED:
+			stock_id = GTK_STOCK_MEDIA_PAUSE;
+			break;
+		default:
+			stock_id = NULL;
+	}
+
+	g_object_set (renderer, "stock-id", stock_id, NULL);
 }
 
 static void
@@ -1252,7 +1212,7 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 
 	/* the model */
 	model = GTK_TREE_MODEL (gtk_list_store_new (NUM_COLS,
-				G_TYPE_BOOLEAN,
+				G_TYPE_INT,
 				G_TYPE_STRING,
 				G_TYPE_STRING,
 				G_TYPE_BOOLEAN,
@@ -1451,29 +1411,16 @@ init_config (TotemPlaylist *playlist)
 
 static void
 totem_playlist_entry_parsed (TotemPlParser *parser,
-		const char *uri, const char *title,
-		const char *genre, TotemPlaylist *playlist)
+			     const char *uri,
+			     GHashTable *metadata,
+			     TotemPlaylist *playlist)
 {
+	const char *title;
+
+	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
 	totem_playlist_add_one_mrl (playlist, uri, title);
 }
 
-static void
-totem_playlist_init (TotemPlaylist *playlist)
-{
-	playlist->_priv = g_new0 (TotemPlaylistPrivate, 1);
-	playlist->_priv->parser = totem_pl_parser_new ();
-
-	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "dvd:");
-	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "cdda:");
-	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "vcd:");
-	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "cd:");
-
-	g_signal_connect (G_OBJECT (playlist->_priv->parser),
-			"entry",
-			G_CALLBACK (totem_playlist_entry_parsed),
-			playlist);
-}
-
 static void
 totem_playlist_finalize (GObject *object)
 {
@@ -1494,23 +1441,30 @@ totem_playlist_finalize (GObject *object)
 	G_OBJECT_CLASS (totem_playlist_parent_class)->finalize (object);
 }
 
-GtkWidget*
-totem_playlist_new (void)
+static void
+totem_playlist_init (TotemPlaylist *playlist)
 {
-	TotemPlaylist *playlist;
-	GtkWidget *container, *item;
+	GtkWidget *container;
 	char *filename;
 
-	playlist = TOTEM_PLAYLIST (g_object_new (TOTEM_TYPE_PLAYLIST, NULL));
+	playlist->_priv = g_new0 (TotemPlaylistPrivate, 1);
+	playlist->_priv->parser = totem_pl_parser_new ();
 
-	playlist->_priv->xml = totem_interface_load ("playlist.glade",
-			_("playlist"), TRUE, NULL);
+	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "dvd:");
+	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "cdda:");
+	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "vcd:");
+	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "cd:");
+	totem_pl_parser_add_ignored_scheme (playlist->_priv->parser, "dvb:");
+
+	g_signal_connect (G_OBJECT (playlist->_priv->parser),
+			"entry-parsed",
+			G_CALLBACK (totem_playlist_entry_parsed),
+			playlist);
+
+	playlist->_priv->xml = totem_interface_load ("playlist.ui", TRUE, NULL, playlist);
 
 	if (playlist->_priv->xml == NULL)
-	{
-		totem_playlist_finalize (G_OBJECT (playlist));
-		return NULL;
-	}
+		return;
 
 	/* popup menu */
 	playlist->_priv->action_group = 
@@ -1527,41 +1481,17 @@ totem_playlist_new (void)
 	filename = totem_interface_get_full_path ("playlist-ui.xml");
 	if (!gtk_ui_manager_add_ui_from_file (playlist->_priv->ui_manager, filename, NULL)) {
 		totem_playlist_finalize (G_OBJECT (playlist));
-		return NULL;
+		return;
 	}
 	g_free (filename);
 
-	/* Connect the buttons */
-	item = glade_xml_get_widget (playlist->_priv->xml, "add_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (totem_playlist_add_files),
-			playlist);
-	item = glade_xml_get_widget (playlist->_priv->xml, "remove_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (playlist_remove_button_clicked),
-			playlist);
-	item = glade_xml_get_widget (playlist->_priv->xml, "save_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (totem_playlist_save_files),
-			playlist);
-	item = glade_xml_get_widget (playlist->_priv->xml, "up_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (totem_playlist_up_files),
-			playlist);
-	item = glade_xml_get_widget (playlist->_priv->xml, "down_button");
-	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (totem_playlist_down_files),
-			playlist);
-
 	gtk_widget_add_events (GTK_WIDGET (playlist), GDK_KEY_PRESS_MASK);
 	g_signal_connect (G_OBJECT (playlist), "key_press_event",
 			G_CALLBACK (totem_playlist_key_press), playlist);
 
 	/* Reparent the vbox */
-	item = glade_xml_get_widget (playlist->_priv->xml, "dialog-vbox1");
-	container = glade_xml_get_widget (playlist->_priv->xml, "vbox4");
+	container = GTK_WIDGET (gtk_builder_get_object (playlist->_priv->xml, "vbox4"));
 	g_object_ref (container);
-	gtk_container_remove (GTK_CONTAINER (item), container);
 	gtk_box_pack_start (GTK_BOX (playlist),
 			container,
 			TRUE,       /* expand */
@@ -1569,8 +1499,8 @@ totem_playlist_new (void)
 			0);         /* padding */
 	g_object_unref (container);
 
-	playlist->_priv->treeview = glade_xml_get_widget
-		(playlist->_priv->xml, "treeview1");
+	playlist->_priv->treeview = GTK_WIDGET (gtk_builder_get_object
+		(playlist->_priv->xml, "treeview1"));
 	init_treeview (playlist->_priv->treeview, playlist);
 	playlist->_priv->model = gtk_tree_view_get_model
 		(GTK_TREE_VIEW (playlist->_priv->treeview));
@@ -1580,6 +1510,20 @@ totem_playlist_new (void)
 
 	gtk_widget_show_all (GTK_WIDGET (playlist));
 
+
+}
+
+GtkWidget*
+totem_playlist_new (void)
+{
+	TotemPlaylist *playlist;
+
+	playlist = TOTEM_PLAYLIST (g_object_new (TOTEM_TYPE_PLAYLIST, NULL));
+	if (playlist->_priv->xml == NULL || playlist->_priv->ui_manager == NULL) {
+		totem_playlist_finalize (G_OBJECT (playlist));
+		return NULL;
+	}
+
 	return GTK_WIDGET (playlist);
 }
 
@@ -1601,16 +1545,16 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 		filename_for_display = g_strdup (display_name);
 	}
 
-	uri = totem_playlist_create_full_path (mrl);
+	uri = totem_create_full_path (mrl);
 
 	D("totem_playlist_add_one_mrl (): %s %s %s\n",
-				filename_for_display, uri, display_name);
+				filename_for_display, uri ? uri : "(null)", display_name);
 
 	store = GTK_LIST_STORE (playlist->_priv->model);
 	gtk_list_store_insert_with_values (store, &iter, G_MAXINT32,
-			PLAYING_COL, FALSE,
+			PLAYING_COL, TOTEM_PLAYLIST_STATUS_NONE,
 			FILENAME_COL, filename_for_display,
-			URI_COL, uri,
+			URI_COL, uri ? uri : mrl,
 			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
 			-1);
 
@@ -1809,8 +1753,8 @@ totem_playlist_clear_with_gnome_vfs_volume (TotemPlaylist *playlist,
 			volume);
 }
 
-char
-*totem_playlist_get_current_mrl (TotemPlaylist *playlist)
+char *
+totem_playlist_get_current_mrl (TotemPlaylist *playlist)
 {
 	GtkTreeIter iter;
 	char *path;
@@ -1832,8 +1776,8 @@ char
 	return path;
 }
 
-char
-*totem_playlist_get_current_title (TotemPlaylist *playlist, gboolean *custom)
+char *
+totem_playlist_get_current_title (TotemPlaylist *playlist, gboolean *custom)
 {
 	GtkTreeIter iter;
 	char *path;
@@ -1856,6 +1800,30 @@ char
 	return path;
 }
 
+char *
+totem_playlist_get_title (TotemPlaylist *playlist, guint index)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	char *title;
+
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), NULL);
+
+	path = gtk_tree_path_new_from_indices (index, -1);
+
+	gtk_tree_model_get_iter (playlist->_priv->model,
+				 &iter, path);
+
+	gtk_tree_path_free (path);
+
+	gtk_tree_model_get (playlist->_priv->model,
+			    &iter,
+			    FILENAME_COL, &title,
+			    -1);
+
+	return title;
+}
+
 gboolean
 totem_playlist_get_current_metadata (TotemPlaylist *playlist,
 				     char         **artist,
@@ -1981,7 +1949,7 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean f
 }
 
 gboolean
-totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
+totem_playlist_set_playing (TotemPlaylist *playlist, TotemPlaylistStatus state)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
@@ -2187,9 +2155,11 @@ totem_playlist_get_current (TotemPlaylist *playlist)
 
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), 0);
 
+	if (playlist->_priv->current == NULL)
+		return 0;
 	path = gtk_tree_path_to_string (playlist->_priv->current);
 	if (path == NULL)
-		return -1;
+		return 0;
 
 	index = g_ascii_strtod (path, NULL);
 	g_free (path);
