@@ -39,8 +39,6 @@ static GtkFileFilter *filter_supported = NULL;
 static GtkFileFilter *filter_audio = NULL;
 static GtkFileFilter *filter_video = NULL;
 
-static char *xdg_user_dir_lookup (const char *type);
-
 gboolean
 totem_playing_dvd (const char *uri)
 {
@@ -82,7 +80,12 @@ totem_dot_dir (void)
 char *
 totem_pictures_dir (void)
 {
-	return xdg_user_dir_lookup ("PICTURES");
+	const char *dir;
+
+	dir = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+	if (dir == NULL)
+		return NULL;
+	return g_strdup (dir);
 }
 
 gboolean
@@ -204,8 +207,8 @@ static const char subtitle_ext[][4] = {
 char *
 totem_uri_get_subtitle_uri (const char *uri)
 {
-	char *suffix, *subtitle;
-	guint len, i;
+	char *subtitle;
+	guint len, i, suffix;
 	GnomeVFSURI *vfsuri;
 
 	if (g_str_has_prefix (uri, "http") != FALSE) {
@@ -218,19 +221,37 @@ totem_uri_get_subtitle_uri (const char *uri)
 		return NULL;
 	gnome_vfs_uri_unref (vfsuri);
 
+	/* Has the user specified a subtitle file manually? */
 	if (strstr (uri, "#subtitle:") != NULL) {
 		return NULL;
 	}
 
+        /* Find the filename suffix delimiter */
 	len = strlen (uri);
-	if (uri[len-4] != '.') {
+	for (suffix = len-1; suffix > 0; suffix--) {
+		if (uri[suffix] == G_DIR_SEPARATOR ||
+                    (uri[suffix] == '/')) {
+			/* This filename has no extension, we'll need to 
+			 * add one */
+			suffix = len;
+			break;
+		}
+		if (uri[suffix] == '.') {
+			/* Found our extension marker */
+			break;
+		}
+	}
+        if (suffix < 0) {
 		return NULL;
 	}
 
-	subtitle = g_strdup (uri);
-	suffix = subtitle + len - 4;
+	/* Generate a subtitle string with room at the end to store the
+	 * 3 character extensions we want to search for */
+	subtitle = g_strdup_printf ("%.*s.???", suffix, uri);
+	
+	/* Search for any files with one of our known subtitle extensions */
 	for (i = 0; i < G_N_ELEMENTS(subtitle_ext) ; i++) {
-		memcpy (suffix + 1, subtitle_ext[i], 3);
+		memcpy (subtitle + suffix + 1, subtitle_ext[i], 3);
 
 		vfsuri = gnome_vfs_uri_new (subtitle);
 		if (vfsuri != NULL) {
@@ -334,9 +355,9 @@ totem_destroy_file_filters (void)
 	}
 }
 
-static const char *dir_types[] = {
-	"VIDEOS",
-	"MUSIC"
+static const GUserDirectory dir_types[] = {
+	G_USER_DIRECTORY_VIDEOS,
+	G_USER_DIRECTORY_MUSIC
 };
 
 static void
@@ -344,28 +365,26 @@ totem_add_default_dirs (GtkFileChooser *dialog)
 {
 	guint i;
 	for (i = 0; i < G_N_ELEMENTS (dir_types); i++) {
-		char *dir;
+		const char *dir;
 
-		dir = xdg_user_dir_lookup (dir_types[i]);
+		dir = g_get_user_special_dir (dir_types[i]);
 		if (dir == NULL)
 			continue;
 		gtk_file_chooser_add_shortcut_folder (dialog, dir, NULL);
-		g_free (dir);
 	}
 }
 
 void
 totem_add_pictures_dir (GtkWidget *chooser)
 {
-	char *dir;
+	const char *dir;
 
 	g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser) != FALSE);
 
-	dir = xdg_user_dir_lookup ("PICTURES");
+	dir = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
 	if (dir == NULL)
 		return;
 	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (chooser), dir, NULL);
-	g_free (dir);
 }
 
 GSList *
@@ -439,124 +458,3 @@ totem_add_files (GtkWindow *parent, const char *path)
 	return filenames;
 }
 
-/* Copied from xdg-user-dir-lookup.c */
-#include <stdlib.h>
-
-static char *
-xdg_user_dir_lookup (const char *type)
-{
-  FILE *file;
-  char *home_dir, *config_home, *config_file;
-  char buffer[512];
-  char *user_dir;
-  char *p, *d;
-  int len;
-  int relative;
-  
-  home_dir = getenv ("HOME");
-
-  if (home_dir == NULL)
-    return strdup ("/tmp");
-
-  config_home = getenv ("XDG_CONFIG_HOME");
-  if (config_home == NULL || config_home[0] == 0)
-    {
-      config_file = malloc (strlen (home_dir) + strlen ("/.config/user-dirs.dirs") + 1);
-      strcpy (config_file, home_dir);
-      strcat (config_file, "/.config/user-dirs.dirs");
-    }
-  else
-    {
-      config_file = malloc (strlen (config_home) + strlen ("/user-dirs.dirs") + 1);
-      strcpy (config_file, config_home);
-      strcat (config_file, "/user-dirs.dirs");
-    }
-
-  file = fopen (config_file, "r");
-  free (config_file);
-  if (file == NULL)
-    goto error;
-
-  user_dir = NULL;
-  while (fgets (buffer, sizeof (buffer), file))
-    {
-      /* Remove newline at end */
-      len = strlen (buffer);
-      if (len > 0 && buffer[len-1] == '\n')
-	buffer[len-1] = 0;
-      
-      p = buffer;
-      while (*p == ' ' || *p == '\t')
-	p++;
-      
-      if (strncmp (p, "XDG_", 4) != 0)
-	continue;
-      p += 4;
-      if (strncmp (p, type, strlen (type)) != 0)
-	continue;
-      p += strlen (type);
-      if (strncmp (p, "_DIR", 4) != 0)
-	continue;
-      p += 4;
-
-      while (*p == ' ' || *p == '\t')
-	p++;
-
-      if (*p != '=')
-	continue;
-      p++;
-      
-      while (*p == ' ' || *p == '\t')
-	p++;
-
-      if (*p != '"')
-	continue;
-      p++;
-      
-      relative = 0;
-      if (strncmp (p, "$HOME/", 6) == 0)
-	{
-	  p += 6;
-	  relative = 1;
-	}
-      else if (*p != '/')
-	continue;
-      
-      if (relative)
-	{
-	  user_dir = malloc (strlen (home_dir) + 1 + strlen (p) + 1);
-	  strcpy (user_dir, home_dir);
-	  strcat (user_dir, "/");
-	}
-      else
-	{
-	  user_dir = malloc (strlen (p) + 1);
-	  *user_dir = 0;
-	}
-      
-      d = user_dir + strlen (user_dir);
-      while (*p && *p != '"')
-	{
-	  if ((*p == '\\') && (*(p+1) != 0))
-	    p++;
-	  *d++ = *p++;
-	}
-      *d = 0;
-    }  
-  fclose (file);
-
-  if (user_dir)
-    return user_dir;
-
- error:
-  /* Special case desktop for historical compatibility */
-  if (strcmp (type, "DESKTOP") == 0)
-    {
-      user_dir = malloc (strlen (home_dir) + strlen ("/Desktop") + 1);
-      strcpy (user_dir, home_dir);
-      strcat (user_dir, "/Desktop");
-      return user_dir;
-    }
-  else
-    return strdup (home_dir);
-}
